package dev.peerat.parser.java;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation.Annotable;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.Function.FunctionContainer;
import dev.peerat.parser.java.Variable.VariableContainer;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;


public abstract class ClassBase extends Annotable implements AnnotableBuffer, ClassContainer, FunctionContainer, VariableContainer{
	
	
	private List<Annotation> annotationBuffer;

	private int mod;
	private Token name;
	
	public ClassBase(List<Annotation> annotations, int mod, Token name){
		super(annotations);
		this.mod = mod;
		this.name = name;
	}
	
	public int getModifier(){
		return this.mod;
	}
	
	public Token getName(){
		return this.name;
	}
	
	@Override
	public void addAnnotationBuffer(Annotation annotation){
		if(annotationBuffer == null) annotationBuffer = new ArrayList<>();
		annotationBuffer.add(annotation);
	}
	
	public List<Annotation> getAnnotationBuffer(){
		List<Annotation> list = this.annotationBuffer;
		this.annotationBuffer = null;
		return list;
	}
	
	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor){
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		if(!visitor.canPropagate()) return bag;
		
		List<Annotation> annotations = getAnnotations();
		if(annotations != null)
			for(Annotation annotation : annotations){
				bag.merge(annotation.visit(visitor));
			}
		return bag;
	}
}
