package dev.peerat.parser.java.builder;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Class;
import dev.peerat.parser.java.JavaElement;

public class JavaClassBuilder extends JavaModifiableBuilder<JavaClassBuilder, dev.peerat.parser.java.Class>{

	private List<Annotation> annotations;
	private String name;
	private String extend;
	private String implement;
	
	private List<JavaElement> elements;
	
	public JavaClassBuilder annotate(JavaAnnotationBuilder annotation){
		if(this.annotations == null) this.annotations = new ArrayList<>();
		this.annotations.add(annotation.build());
		return this;
	}
	
	public JavaClassBuilder setName(String name){
		this.name = name;
		return this;
	}
	
	public JavaClassBuilder extend(String type){
		this.extend = type;
		return this;
	}
	
	public JavaClassBuilder implement(String type){
		if(implement == null) this.implement = type;
		else this.implement+=","+type;
		return this;
	}
	
	public <T extends JavaElement> JavaClassBuilder element(JavaBuilder<T> builder){
		if(this.elements == null) this.elements = new ArrayList<>();
		this.elements.add(builder.build());
		return this;
	}
	
	public JavaClassBuilder normalize(dev.peerat.parser.java.Class clazz){
		normalizeAnnotations(clazz);
		normalizeSignature(clazz);
		//elements
		return this;
	}
	
	public JavaClassBuilder normalizeAnnotations(dev.peerat.parser.java.Class clazz){
		List<Annotation> annotations = clazz.getAnnotations();
		if(annotations != null && annotations.size() > 0){
			if(this.annotations == null) this.annotations = new ArrayList<>();
			for(Annotation annotation : annotations){
				this.annotations.add(JavaBuilder.ofAnnotation().normalize(annotation).build());
			}
		}
		return this;
	}
	
	public JavaClassBuilder normalizeSignature(dev.peerat.parser.java.Class clazz){
		this.mod = clazz.getModifier();
		this.name = clazz.getName().getValue();
		if(clazz.getExtension() != null) this.extend = clazz.getExtension().getValue();
		if(clazz.getImplementations() != null) this.implement = this.implement == null ? clazz.getImplementations().getValue() : ","+clazz.getImplementations().getValue();
		return this;
	}
	
	public JavaClassBuilder clearAnnotations(){
		this.annotations = null;
		return this;
	}
	
	@Override
	public Class build(){
		dev.peerat.parser.java.Class result = new dev.peerat.parser.java.Class(this.annotations, this.mod, new Token(0,0,name, null), extend == null ? null : new Token(0,0,extend, null), implement == null ? null : new Token(0,0, implement, null));
		if(elements != null) result.getElements().addAll(elements);
		return result;
	}

}
