package dev.peerat.parser.java.printer;

import java.io.BufferedWriter;
import java.util.HashMap;
import java.util.Map;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.Import;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.JavaFile;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.operation.ReturnOperation;
import dev.peerat.parser.java.printer.JavaOperationPrinter.JavaOperationReturnPrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaArrayAccessValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaArrayValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaBiValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaCastValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaInstanceValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaLambdaValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaMethodCallValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaModifierValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaStaticValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaTriValuePrinter;
import dev.peerat.parser.java.printer.JavaValuePrinter.JavaVariableAccessValuePrinter;
import dev.peerat.parser.java.value.ArrayAccessValue;
import dev.peerat.parser.java.value.ArrayValue;
import dev.peerat.parser.java.value.BiValue;
import dev.peerat.parser.java.value.CastValue;
import dev.peerat.parser.java.value.InstanceValue;
import dev.peerat.parser.java.value.LambdaValue;
import dev.peerat.parser.java.value.MethodCallValue;
import dev.peerat.parser.java.value.ModifierValue;
import dev.peerat.parser.java.value.StaticValue;
import dev.peerat.parser.java.value.TriValue;
import dev.peerat.parser.java.value.VariableAccessValue;

public abstract class JavaPrinter<T extends JavaElement>{
	
	public static JavaPrintProvider getProvider(){
		JavaPrintProvider provider = new JavaPrintProvider();
		provider.register(ArrayAccessValue.class, new JavaArrayAccessValuePrinter());
		provider.register(ArrayValue.class, new JavaArrayValuePrinter());
		provider.register(BiValue.class, new JavaBiValuePrinter());
		provider.register(CastValue.class, new JavaCastValuePrinter());
		provider.register(InstanceValue.class, new JavaInstanceValuePrinter());
		provider.register(LambdaValue.class, new JavaLambdaValuePrinter());
		provider.register(MethodCallValue.class, new JavaMethodCallValuePrinter());
		provider.register(ModifierValue.class, new JavaModifierValuePrinter());
		provider.register(StaticValue.class, new JavaStaticValuePrinter());
		provider.register(TriValue.class, new JavaTriValuePrinter());
		provider.register(VariableAccessValue.class, new JavaVariableAccessValuePrinter());
		
		provider.register(ReturnOperation.class, new JavaOperationReturnPrinter());
		
		provider.register(Import.class, new JavaImportPrinter());
		provider.register(Annotation.class, new JavaAnnotationPrinter());
		provider.register(Variable.class, new JavaVariablePrinter());
		provider.register(Function.class, new JavaFunctionPrinter());
		provider.register(dev.peerat.parser.java.Class.class, new JavaClassPrinter());
		provider.register(JavaFile.class, new JavaFilePrinter());
		return provider;
	}
	
	public abstract void print(T element, Writer writer, String buffer, JavaPrintProvider provider) throws Exception;
	
	public static class JavaPrintProvider{
		
		private Map<Class<?>, JavaPrinter<?>> map;
		
		public JavaPrintProvider(){
			this.map = new HashMap<>();
		}
		
		public void register(Class<?> type, JavaPrinter<?> printer){
			this.map.put(type, printer);
		}
		
		public <T extends JavaElement> JavaPrinter<T> getPrinter(T type){
			return (JavaPrinter<T>) getPrinter(type.getClass());
		}
		
		public <T extends JavaElement> JavaPrinter<T> getPrinter(Class<T> type){
			return (JavaPrinter<T>) this.map.get(type);
		}
	}
	
	public static class Writer{
		
		private BufferedWriter writer;
		
		public Writer(BufferedWriter writer){
			this.writer = writer;
		}
		
		public void write(String value) throws Exception{
			this.writer.write(value);
		}
		
		public void write(Token token) throws Exception{
			write(token.getValue());
		}
	}
}
