package dev.peerat.parser.java.tree;

import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;
import java.util.function.BiFunction;

import dev.peerat.parser.Bag;
import dev.peerat.parser.Token;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.Function.FunctionContainer;
import dev.peerat.parser.state.BuilderStateTree;
import dev.peerat.parser.state.InitialStateTree;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class FunctionTree extends SyntaxTree<JavaElement> {

	public FunctionTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new InitialStateTree<>();
	}

	@Override
	public void init() {
		StateTree<JavaElement> modifier = registery.getNode(ModifierTree.class);
		StateTree<JavaElement> type = registery.getNode(TypeTree.class);
		StateTree<JavaElement> gen = registery.getNode(GenericTree.class);
		StateTree<JavaElement> value = registery.getNode(ValueTree.class);
		StateTree<JavaElement> annotation = registery.getNode(AnnotationTree.class);
		StateTree<JavaElement> function_container = registery.getNode(FunctionContainerTree.class);
		
		// FUNCTION
		BiFunction<JavaElement, Bag, JavaElement> function_builder = (parent, bag) -> {
			buildVariable(bag);
			Integer mod = bag.get("mod");
			List<Annotation> annotations = null;
			if (parent instanceof AnnotableBuffer)
				annotations = (((AnnotableBuffer) parent).getAnnotationBuffer());
			Function function = new Function(annotations, mod == null ? 0 : mod, bag.get("generic"), bag.get("type"),
					bag.get("name"), bag.get("param"), bag.get("throws"));
			if (parent instanceof FunctionContainer)
				((FunctionContainer) parent).addFunction(function);
			return function;
		};

		InitialStateTree<JavaElement> function = (InitialStateTree<JavaElement>) this.current;
		function.multiple(annotation);
		BuilderStateTree<JavaElement, JavaElement> function_static = function
				.then((validator) -> validator.validate((token) -> token.getValue().equals("static"))
						&& validator.validate((token) -> token.getValue().equals("{")))
				.end((parent, bag) -> {
					Function build = new Function((((AnnotableBuffer) parent).getAnnotationBuffer()), Modifier.STATIC,
							null, null);
					if (parent instanceof FunctionContainer)
						((FunctionContainer) parent).addFunction(build);
					return build;
				});
		function_static.multiple(function_container);
		function_static.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end();
		StateTree<JavaElement> function_mod = function.then(new RedirectStateTree<>(modifier, (global, local) -> {
			Integer mod = global.get("mod");
			if (mod == null)
				global.set("mod", local.get());
			else
				global.set("mod", mod + local.<Integer>get());
		}));
		StateTree<JavaElement> function_generic = function
				.then(new RedirectStateTree<>(gen, (global, local) -> global.set("generic", local.get())));
		StateTree<JavaElement> function_type = function
				.then(new RedirectStateTree<>(type, (global, local) -> global.set("type", local.get())));
		function_mod.then(function_mod);
		function_mod.then(function_generic);
		function_mod.then(function_type);
		function_generic.then(function_type);

		StateTree<JavaElement> function_name = function.then((validator) -> validator
				.validate((token) -> token.getType().equals(TokenType.NAME), (bag, token) -> bag.set("name", token)));
		function_type.then(function_name);
		function_mod.then(function_name);
		StateTree<JavaElement> function_begin = function_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")));
		StateTree<JavaElement> function_end = function_begin
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")));
		function_end.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
				.end(function_builder).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end();
		function_end.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end();
		function_end.then((validator) -> validator.validate((token) -> token.getValue().equals("default")))
				.then(new RedirectStateTree<>(value, (global, local) -> System.out.println("TODO DEFAULT IN FUNCTION")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end(function_builder);

		StateTree<JavaElement> function_throws = function_end
				.then((validator) -> validator.validate((token) -> token.getValue().equals("throws")))
				.then(new RedirectStateTree<>(type, (global, local) -> {
					List<Token> throwables = global.get("throws");
					if (throwables == null) {
						throwables = new ArrayList<>();
						global.set("throws", throwables);
					}
					throwables.add(local.get());
				}));
		BuilderStateTree<JavaElement, JavaElement> function_start_throws = function_throws
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{"))).end(function_builder);
		function_start_throws.multiple(function_container);
		function_start_throws.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end();
		function_throws.then((validator) -> validator.validate((token) -> token.getValue().equals(",")))
				.then(function_throws);
		function_throws.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
				.end(function_builder);

		StateTree<JavaElement> function_arg_mod = function_begin.then((validator) -> validator
				.validate((token) -> token.getValue().equals("final"), (bag, token) -> bag.set("arg_mod", true)));
		StateTree<JavaElement> function_arg_type = function_begin
				.then(new RedirectStateTree<>(type, (global, local) -> global.set("arg_type", local.get())));
		function_arg_mod.then(function_arg_type);
		StateTree<JavaElement> function_arg_type_array_begin = function_arg_type
				.then((validator) -> validator.validate((token) -> token.getValue().equals("["),
						(bag, token) -> bag.set("arg_type", bag.<Token>get("arg_type").concat(token))));
		StateTree<JavaElement> function_arg_type_array_end = function_arg_type_array_begin
				.then((validator) -> validator.validate((token) -> token.getValue().equals("]"),
						(bag, token) -> bag.set("arg_type", bag.<Token>get("arg_type").concat(token))));
		function_arg_type_array_end.then(function_arg_type_array_begin);
		StateTree<JavaElement> function_arg_name = function_arg_type
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME),
						(bag, token) -> bag.set("arg_name", token)));
		function_arg_type_array_end.then(function_arg_name);
		function_arg_type
				.then((validator) -> validator.validate((token) -> token.getValue().equals("."))
						&& validator.validate((token) -> token.getValue().equals(".")) && validator.validate(
								(token) -> token.getValue().equals("."), (bag, token) -> bag.set("arg_elips", true)))
				.then(function_arg_name);
		StateTree<JavaElement> function_arg_split = function_arg_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals(","), (bag, token) -> {
					buildVariable(bag);
				}));
		function_arg_split.then(function_arg_mod);
		function_arg_split.then(function_arg_type);
		function_arg_name.then(function_end);
	}

	// TAKE ANNOTATIONS ???????
	private static void buildVariable(Bag bag) {
		if (!bag.has("arg_name"))
			return;

		List<Variable> parameters = bag.get("param");
		if (parameters == null) {
			parameters = new ArrayList<>();
			bag.set("param", parameters);
		}

		Integer mod = bag.get("arg_mod");

		Variable variable = new Variable(null, mod == null ? 0 : mod, bag.<Token>get("arg_type"),
				bag.<Token>get("arg_name"), bag.get("arg_elips") == Boolean.TRUE);
		bag.remove("arg_name");
		bag.remove("arg_mod");
		bag.remove("arg_type");
		bag.remove("arg_name");
		bag.remove("arg_elips");

		parameters.add(variable);
	}

}
