package dev.peerat.parser.java.visitor;

import java.util.function.Predicate;

import dev.peerat.parser.java.Variable;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaVariableVisitor extends JavaModifiableVisitor<Variable, JavaVariableVisitor>{
	
	private Predicate<String> typeFilter;
	private Predicate<String> nameFilter;
	
	public JavaVariableVisitor(boolean propagate){
		super(Variable.class, propagate);
	}
	
	public JavaVariableVisitor typeFilter(Predicate<String> validator){
		this.typeFilter = (this.typeFilter == null) ? validator : this.typeFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor nameFilter(Predicate<String> validator){
		this.nameFilter = (this.nameFilter == null) ? validator : this.nameFilter.and(validator);
		return this;
	}
	
	@Override
	public VisitorBag visitElement(Variable element){
		VisitorBag bag = new VisitorBag();
		
		if(typeFilter != null){
			//ellipse ?
			if(!typeFilter.test(element.getType().getValue())) return bag;
		}
		
		if(nameFilter != null){
			if(!nameFilter.test(element.getName().getValue())) return bag;
		}
		
		if(this.modifierFilter != null){
			if(!this.modifierFilter.test(element.getModifier())) return bag;
		}
		
		bag.compute(element);
		return bag;
	}
	
	

}
