package dev.peerat.parser.java.visitor.value;

import dev.peerat.parser.java.value.CastValue;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.java.value.Value;
import java.util.function.Predicate;

public class JavaCastValueVisitor extends JavaVisitor<CastValue> {
	private Predicate<String> typePredicate;
	private Visitor<Value> valuePredicate;

	public JavaCastValueVisitor(boolean propagate) {
		super(CastValue.class, propagate);
	}

	public JavaCastValueVisitor type(Predicate<String> checker) {
		typePredicate = checker;
		return this;
	}

	public JavaCastValueVisitor value(Visitor<Value> visitor) {
		valuePredicate = visitor;
		return this;
	}

	@Override
	public VisitorBag visitElement(CastValue element) {
		VisitorBag bag = new VisitorBag();
		
		if(typePredicate != null){
			if(!typePredicate.test(element.getType().getValue())) return bag;
		}
		
		if(valuePredicate != null){
			if(!valuePredicate.canVisit(element.getValue().getClass())) return bag;
			VisitorBag visited = valuePredicate.visit(element.getValue());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		bag.compute(element);
		return bag;
	}
}
