package dev.peerat.parser.java.element.value;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.jupiter.api.Assertions.assertEquals;

import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.JavaFile;
import dev.peerat.parser.java.element.BaseElementTests;
import dev.peerat.parser.java.operation.IfOperation;
import dev.peerat.parser.java.operation.Operation;
import dev.peerat.parser.java.value.BiValue;
import dev.peerat.parser.java.value.StaticValue;
import dev.peerat.parser.java.value.Value;

public class TypeOfValuesTests extends BaseElementTests{
	
	{
		register(
				"package be.jeffcheasey88;"
				+ ""
				+ "public static class Test{ "
				+ "    void test(){ "
				+ "        if(a){}"
				+ "}"
				+ "}",
				(javafile) -> {
					StaticValue value = gotoValue(javafile);
					assertToken("a", value.getToken());
				});
		
		register(
				"package be.jeffcheasey88;"
				+ ""
				+ "public static class Test{ "
				+ "    void test(){ "
				+ "        if(a > b){}"
				+ "}"
				+ "}",
				(javafile) -> {
					BiValue value = gotoValue(javafile);
					StaticValue left = (StaticValue) value.left();
					StaticValue right = (StaticValue) value.right(); //generic?
					assertEquals("a", left.getToken().getValue());
					assertEquals("b", right.getToken().getValue());
					assertEquals(">", value.getAction());
				});
		
		register(
				"package be.jeffcheasey88;"
				+ ""
				+ "public static class Test{ "
				+ "    void test(){ "
				+ "        if(((a) > (b))){}"
				+ "}"
				+ "}",
				(javafile) -> {
					BiValue value = gotoValue(javafile);
					StaticValue left = (StaticValue) value.left();
					StaticValue right = (StaticValue) value.right(); //generic?
					assertEquals("a", left.getToken().getValue());
					assertEquals("b", right.getToken().getValue());
					assertEquals(">", value.getAction());
				});
		
		register(
				"package be.jeffcheasey88;"
				+ ""
				+ "public static class Test{ "
				+ "    void test(){ "
				+ "        if(a && b == c){}"
				+ "}"
				+ "}",
				(javafile) -> {
					BiValue value = gotoValue(javafile);
					printObject(value);
					StaticValue left = (StaticValue) value.left();
					BiValue right = (BiValue) value.right();
					
					StaticValue rightLeft = (StaticValue) right.left();
					StaticValue rightRight = (StaticValue) right.right();
					
					assertToken("a", left.getToken());
					assertEquals("&&", value.getAction());
					assertToken("b", rightLeft.getToken());
					
					assertEquals("==", right.getAction());
					assertToken("c", rightRight.getToken());
				});
		
		register(
				"package be.jeffcheasey88;"
				+ ""
				+ "public static class Test{ "
				+ "    void test(){ "
				+ "        if((a && b) == c){}"
				+ "}"
				+ "}",
				(javafile) -> {
					BiValue value = gotoValue(javafile);
					printObject(value);
					BiValue left = (BiValue) value.left();
					StaticValue right = (StaticValue) value.right();
					
					StaticValue leftLeft = (StaticValue) left.left();
					StaticValue leftRight = (StaticValue) left.right();
					
					assertToken("a", leftLeft.getToken());
					assertEquals("&&", left.getAction());
					assertToken("b", leftRight.getToken());
					
					assertEquals("==", value.getAction());
					assertToken("c", right.getToken());
				});
	}
	
	private <T extends Value> T gotoValue(JavaFile javafile){
		Function function = checkFunction(checkClass(javafile).getElements().get(0));
		assertEquals(1, function.getElements().size());
		Operation op = checkOperation(function.getElements().get(0));
		assertTrue(op instanceof IfOperation);
		IfOperation condition = (IfOperation)op;
		Value value = condition.getCondition();
		assertNotNull(value);
		return (T) value;
	}

}
